@@ -1,7 +1,7 @@
 #ifndef QT_WAYLAND_SERVER_HARDWARE_INTEGRATION
 #define QT_WAYLAND_SERVER_HARDWARE_INTEGRATION
 
-#include "wayland-server.h"
+#include "wayland-server-core.h"
 #include <QtWaylandCompositor/private/wayland-hardware-integration-server-protocol.h>
 #include <QByteArray>
 #include <QMultiMap>
@@ -17,6 +17,7 @@
 QT_BEGIN_NAMESPACE
 QT_WARNING_PUSH
 QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
+QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")
 
 #if !defined(Q_WAYLAND_SERVER_HARDWARE_INTEGRATION_EXPORT)
 #  if defined(QT_SHARED)
@@ -44,9 +45,10 @@ namespace QtWaylandServer {
             virtual ~Resource() {}
 
             qt_hardware_integration *hardware_integration_object;
+            qt_hardware_integration *object() { return hardware_integration_object; } 
             struct ::wl_resource *handle;
 
-            struct ::wl_client *client() const { return handle->client; }
+            struct ::wl_client *client() const { return wl_resource_get_client(handle); }
             int version() const { return wl_resource_get_version(handle); }
 
             static Resource *fromResource(struct ::wl_resource *resource);
@@ -88,6 +90,7 @@ namespace QtWaylandServer {
     private:
         static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
         static void destroy_func(struct ::wl_resource *client_resource);
+        static void display_destroy_func(struct ::wl_listener *listener, void *data);
 
         Resource *bind(struct ::wl_client *client, uint32_t id, int version);
         Resource *bind(struct ::wl_resource *handle);
@@ -96,6 +99,10 @@ namespace QtWaylandServer {
         Resource *m_resource;
         struct ::wl_global *m_global;
         uint32_t m_globalVersion;
+        struct DisplayDestroyedListener : ::wl_listener {
+            qt_hardware_integration *parent;
+        };
+        DisplayDestroyedListener m_displayDestroyedListener;
     };
 }
 
